@@ -3,9 +3,11 @@ from __future__ import annotations
 import asyncio
 import datetime
 import functools
-from asyncio import AbstractEventLoop
+from asyncio import AbstractEventLoop, TimerHandle
 from typing import Callable, Coroutine, Optional, Tuple, TypeVar
 
+from roborock import RoborockException
+
 T = TypeVar("T")
 DEFAULT_TIME_ZONE: Optional[datetime.tzinfo] = datetime.datetime.now().astimezone().tzinfo
 
@@ -23,6 +25,25 @@ def get_running_loop_or_create_one() -> AbstractEventLoop:
     return loop
 
 
+def parse_datetime_to_roborock_datetime(
+    start_datetime: datetime.datetime, end_datetime: datetime.datetime
+) -> Tuple[datetime.datetime, datetime.datetime]:
+    now = datetime.datetime.now(DEFAULT_TIME_ZONE)
+    start_datetime = start_datetime.replace(
+        year=now.year, month=now.month, day=now.day, second=0, microsecond=0, tzinfo=DEFAULT_TIME_ZONE
+    )
+    end_datetime = end_datetime.replace(
+        year=now.year, month=now.month, day=now.day, second=0, microsecond=0, tzinfo=DEFAULT_TIME_ZONE
+    )
+    if start_datetime > end_datetime:
+        end_datetime += datetime.timedelta(days=1)
+    elif end_datetime < now:
+        start_datetime += datetime.timedelta(days=1)
+        end_datetime += datetime.timedelta(days=1)
+
+    return start_datetime, end_datetime
+
+
 def parse_time_to_datetime(
     start_time: datetime.time, end_time: datetime.time
 ) -> Tuple[datetime.datetime, datetime.datetime]:
@@ -34,13 +55,7 @@ def parse_time_to_datetime(
         hour=end_time.hour, minute=end_time.minute, second=0, microsecond=0
     )
 
-    if start_datetime > end_datetime:
-        end_datetime += datetime.timedelta(days=1)
-    elif end_datetime < datetime.datetime.now(DEFAULT_TIME_ZONE):
-        start_datetime += datetime.timedelta(days=1)
-        end_datetime += datetime.timedelta(days=1)
-
-    return start_datetime, end_datetime
+    return parse_datetime_to_roborock_datetime(start_datetime, end_datetime)
 
 
 def run_sync():
@@ -56,27 +71,29 @@ def run_sync():
     return decorator
 
 
-class CacheableResult:
-    last_run_result = None
+class RepeatableTask:
+    def __init__(self, loop: AbstractEventLoop, callback: Callable[[], Coroutine], interval: int):
+        self.loop = loop
+        self.callback = callback
+        self.interval = interval
+        self._task: Optional[TimerHandle] = None
 
-
-RT = TypeVar("RT", bound=Callable[..., Coroutine])
-
-
-def fallback_cache(func: RT) -> RT:
-    cache = CacheableResult()
-
-    @functools.wraps(func)
-    async def wrapped(*args, **kwargs):
+    async def _run_task(self):
+        response = None
         try:
-            last_run_result = await func(*args, **kwargs)
-            cache.last_run_result = last_run_result
-        except Exception as e:
-            if cache.last_run_result is None:
-                raise e
-            last_run_result = cache.last_run_result
-            cache.last_run_result = None
-            return last_run_result
-        return last_run_result
+            response = await self.callback()
+        except RoborockException:
+            pass
+        self._task = self.loop.call_later(self.interval, self._run_task_soon)
+        return response
 
-    return wrapped  # type: ignore
+    def _run_task_soon(self):
+        asyncio.create_task(self._run_task())
+
+    def cancel(self):
+        if self._task:
+            self._task.cancel()
+
+    async def reset(self):
+        self.cancel()
+        return await self._run_task()
